@@ -138,5 +138,8 @@
 /* Use the included regex.c? */
 #undef USE_GNU_REGEX
 
-/* Do we have gpgm installed? */
-#undef HAVE_GPGM
+/* Where's mixmaster located? */
+#undef MIXMASTER
+
+/* Where are the character set definitions located? */
+#undef CHARMAPS_DIR
