@@ -124,9 +124,13 @@ int mutt_compose_attachment (BODY *a)
       }
       else
       {
+	int r;
+
 	endwin ();
-	mutt_system (command);
-	if (entry->composetypecommand)
+	if ((r = mutt_system (command)) == -1)
+	  mutt_error (_("Error running \"%s\"!"), command);
+	
+	if (r != -1 && entry->composetypecommand)
 	{
 	  BODY *b;
 	  FILE *fp, *tfp;
@@ -248,7 +252,8 @@ int mutt_edit_attachment (BODY *a)
       else
       {
 	endwin ();
-	mutt_system (command);
+	if (mutt_system (command) == -1)
+	  mutt_error (_("Error running \"%s\"!"), command);
       }
     }
   }
@@ -707,11 +712,9 @@ int mutt_save_attachment (FILE *fp, BODY *m, char *path, int flags, HEADER *hdr)
       
       memset (&s, 0, sizeof (s));
       if (flags == M_SAVE_APPEND)
-	s.fpout = fopen (path, "a");
-      else if (flags == M_SAVE_OVERWRITE)
-	s.fpout = fopen (path, "w");
+	s.fpout = safe_fopen (path, "a");
       else
-	s.fpout = safe_fopen (path, "w");
+	s.fpout = fopen (path, "w");
       if (s.fpout == NULL)
       {
 	mutt_perror ("fopen");
@@ -762,7 +765,7 @@ int mutt_save_attachment (FILE *fp, BODY *m, char *path, int flags, HEADER *hdr)
 
 /* returns 0 on success, -1 on error */
 int mutt_decode_save_attachment (FILE *fp, BODY *m, char *path,
-					      int displaying, int flags)
+				 int displaying, int flags)
 {
   STATE s;
   unsigned int saved_encoding = 0;
@@ -770,15 +773,12 @@ int mutt_decode_save_attachment (FILE *fp, BODY *m, char *path,
   HEADER *saved_hdr = NULL;
 
   memset (&s, 0, sizeof (s));
-  s.flags = displaying ? M_DISPLAY : 0;
+  s.flags = (displaying ? M_DISPLAY : 0);
 
   if (flags == M_SAVE_APPEND)
-    s.fpout = fopen (path, "a");
-  else if (flags == M_SAVE_OVERWRITE)
-    s.fpout = fopen (path, "w");
+    s.fpout = safe_fopen (path, "a");
   else
-    s.fpout = safe_fopen (path, "w");
-
+    s.fpout = fopen (path, "w");
   if (s.fpout == NULL)
   {
     perror ("fopen");
@@ -811,9 +811,16 @@ int mutt_decode_save_attachment (FILE *fp, BODY *m, char *path,
     saved_parts = m->parts;
     saved_hdr = m->hdr;
     mutt_parse_part (s.fpin, m);
+
+    /* display a readable version to the user */
+    if (m->noconv)
+      s.flags |= M_CHARCONV;
   }
   else
+  {
     s.fpin = fp;
+    s.flags |= M_CHARCONV;
+  }
 
   mutt_body_handler (m, &s);
 
