@@ -392,20 +392,20 @@ static int count_delete_lines (FILE *fp, BODY *b, long *length, size_t datelen)
 }
 
 /* make a copy of a message
- 
-   fpout	where to write output
-   fpin		where to get input
-   hdr		header of message being copied
-   body		structure of message being copied
-   flags
-	M_CM_NOHEADER	don't copy header
- 	M_CM_PREFIX	quote header and body
- 	M_CM_DECODE	decode message body to text/plain
- 	M_CM_DISPLAY	displaying output to the user
-	M_CM_UPDATE	update structures in memory after syncing
-	M_CM_DECODE_PGP	used for decoding PGP messages
-        M_CM_CHARCONV	perform character set conversion
-   chflags	flags to mutt_copy_header()
+ * 
+ * fpout	where to write output
+ * fpin		where to get input
+ * hdr		header of message being copied
+ * body		structure of message being copied
+ * flags
+ * 	M_CM_NOHEADER	don't copy header
+ * 	M_CM_PREFIX	quote header and body
+ *	M_CM_DECODE	decode message body to text/plain
+ *	M_CM_DISPLAY	displaying output to the user
+ *	M_CM_UPDATE	update structures in memory after syncing
+ *	M_CM_DECODE_PGP	used for decoding PGP messages
+ *	M_CM_CHARCONV	perform character set conversion 
+ * chflags	flags to mutt_copy_header()
  */
 
 int
@@ -501,7 +501,7 @@ _mutt_copy_message (FILE *fpout, FILE *fpin, HEADER *hdr, BODY *body,
       s.flags |= M_WEED;
     if (flags & M_CM_CHARCONV)
       s.flags |= M_CHARCONV;
-
+    
 #ifdef _PGPPATH
     if (flags & M_CM_VERIFY)
       s.flags |= M_VERIFY;
@@ -596,10 +596,8 @@ _mutt_append_message (CONTEXT *dest, FILE *fpin, CONTEXT *src, HEADER *hdr,
     chflags |= CH_FROM;
   chflags |= (dest->magic == M_MAILDIR ? CH_NOSTATUS : CH_UPDATE);
   r = _mutt_copy_message (msg->fp, fpin, hdr, body, flags, chflags);
-  if (mx_commit_message (msg, dest) != 0)
-    r = -1;
-
-  mx_close_message (&msg);
+  if (mx_close_message (&msg) != 0)
+    return (-1);
   return r;
 }
 
