@@ -1,4 +1,3 @@
-/* $Id: copy.h,v 2.0 1998/12/12 08:06:00 roessler Rel $ */
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
@@ -23,7 +22,8 @@
 #define M_CM_DECODE	(1<<2)	/* decode the message body into text/plain */
 #define M_CM_DISPLAY	(1<<3)	/* output is displayed to the user */
 #define M_CM_UPDATE	(1<<4)  /* update structs on sync */
-
+#define M_CM_WEED	(1<<5)  /* weed message/rfc822 attachment headers */
+#define M_CM_CHARCONV	(1<<6)  /* perform character set conversions */
 
 
 #ifdef _PGPPATH
