@@ -1,4 +1,3 @@
-static const char rcsid[]="$Id: curs_lib.c,v 2.2 1998/12/17 00:52:58 roessler Exp $";
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
@@ -54,7 +53,7 @@ event_t mutt_getch (void)
   if (UngetCount)
     return (KeyEvent[--UngetCount]);
 
-  Signals &= ~S_INTERRUPT;
+  SigInt = 0;
 
 #ifdef KEY_RESIZE
   /* ncurses 4.2 sends this when the screen is resized */
@@ -63,7 +62,7 @@ event_t mutt_getch (void)
 #endif /* KEY_RESIZE */
     ch = getch ();
 
-  if (Signals & S_INTERRUPT)
+  if (SigInt)
     mutt_query_exit ();
 
   if(ch == ERR)
@@ -84,7 +83,7 @@ event_t mutt_getch (void)
   return (ch == ctrl ('G') ? err : ret);
 }
 
-int mutt_get_field (/* const */ char *field, char *buf, size_t buflen, int complete)
+int _mutt_get_field (/* const */ char *field, char *buf, size_t buflen, int complete, int multiple, char ***files, int *numfiles)
 {
   int ret;
   int len = mutt_strlen (field); /* in case field==buffer */
@@ -94,7 +93,7 @@ int mutt_get_field (/* const */ char *field, char *buf, size_t buflen, int compl
     CLEARLINE (LINES-1);
     addstr (field);
     mutt_refresh ();
-    ret = mutt_enter_string ((unsigned char *) buf, buflen, LINES-1, len, complete);
+    ret = _mutt_enter_string ((unsigned char *) buf, buflen, LINES-1, len, complete, multiple, files, numfiles);
   }
   while (ret == 1);
   CLEARLINE (LINES-1);
@@ -125,7 +124,8 @@ void mutt_edit_file (const char *editor, const char *data)
   
   endwin ();
   mutt_expand_file_fmt (cmd, sizeof (cmd), editor, data);
-  mutt_system (cmd);
+  if (mutt_system (cmd) == -1)
+    mutt_error (_("Error running \"%s\"!"), cmd);
   keypad (stdscr, TRUE);
   clearok (stdscr, TRUE);
 }
@@ -180,7 +180,7 @@ void mutt_query_exit (void)
   }
   mutt_clear_error();
   mutt_curs_set (-1);
-  Signals &= ~S_INTERRUPT;
+  SigInt = 0;
 }
 
 static void clean_error_buf(void)
@@ -245,23 +245,24 @@ void mutt_show_error (void)
 
 void mutt_endwin (const char *msg)
 {
-
+  if (!option (OPTNOCURSES))
+  {
 #ifdef SLANG_CURSES
-  CLEARLINE (LINES - 1);
-  move (LINES - 1, 0)
+    CLEARLINE (LINES - 1);
+    move (LINES - 1, 0)
 #else
-  move (LINES - 1, COLS - 1);
-  CLEARLINE (LINES - 1);
+      move (LINES - 1, COLS - 1);
+    CLEARLINE (LINES - 1);
 #endif
-
-  attrset (A_NORMAL);
-  mutt_refresh ();
-  endwin ();
-
+    
+    attrset (A_NORMAL);
+    mutt_refresh ();
+    endwin ();
 #ifndef SLANG_CURSES
-  fputc ('\n', stdout);
+    fputc ('\n', stdout);
 #endif
-
+  }
+  
   if (msg)
     puts (msg);
 }
@@ -317,15 +318,20 @@ int mutt_do_pager (const char *banner,
     
     endwin ();
     mutt_expand_file_fmt (cmd, sizeof(cmd), Pager, tempfile);
-    mutt_system (cmd);
+    if (mutt_system (cmd) == -1)
+    {
+      mutt_error (_("Error running \"%s\"!"), cmd);
+      rc = -1;
+    }
+    else
+      rc = 0;
     mutt_unlink (tempfile);
-    rc = 0;
   }
 
   return rc;
 }
 
-int mutt_enter_fname (const char *prompt, char *buf, size_t blen, int *redraw, int buffy)
+int _mutt_enter_fname (const char *prompt, char *buf, size_t blen, int *redraw, int buffy, int multiple, char ***files, int *numfiles)
 {
   event_t ch;
 
@@ -346,7 +352,7 @@ int mutt_enter_fname (const char *prompt, char *buf, size_t blen, int *redraw, i
   {
     mutt_refresh ();
     buf[0] = 0;
-    mutt_select_file (buf, blen, 0);
+    _mutt_select_file (buf, blen, 0, multiple, files, numfiles);
     *redraw = REDRAW_FULL;
   }
   else
@@ -355,7 +361,7 @@ int mutt_enter_fname (const char *prompt, char *buf, size_t blen, int *redraw, i
 
     sprintf (pc, "%s: ", prompt);
     mutt_ungetch (ch.op ? 0 : ch.ch, ch.op ? ch.op : 0);
-    if (mutt_get_field (pc, buf, blen, (buffy ? M_EFILE : M_FILE) | M_CLEAR)
+    if (_mutt_get_field (pc, buf, blen, (buffy ? M_EFILE : M_FILE) | M_CLEAR, multiple, files, numfiles)
 	!= 0)
       buf[0] = 0;
     MAYBE_REDRAW (*redraw);
