@@ -47,35 +47,30 @@
 #include <sys/stat.h>
 #include <errno.h>
 
-static const char No_mailbox_is_open[] = N_("No mailbox is open.");
-static const char There_are_no_messages[] = N_("There are no messages.");
-static const char Mailbox_is_read_only[] = N_("Mailbox is read-only.");
-static const char Function_not_permitted_in_attach_message_mode[] = N_("Function not permitted in attach-message mode.");
-
 #define CHECK_MSGCOUNT if (!Context) \
 	{ \
 	  	mutt_flushinp (); \
-		mutt_error _(No_mailbox_is_open); \
+		mutt_error _("No mailbox is open."); \
 		break; \
 	} \
 	else if (!Context->msgcount) \
 	{ \
 	  	mutt_flushinp (); \
-		mutt_error _(There_are_no_messages); \
+		mutt_error _("There are no messages."); \
 		break; \
 	}
 
 #define CHECK_READONLY if (Context->readonly) \
 			{ \
 			  	mutt_flushinp (); \
-				mutt_error _(Mailbox_is_read_only); \
+				mutt_error _("Mailbox is read-only."); \
 				break; \
 			}
 
 #define CHECK_ATTACH if(option(OPTATTACHMSG)) \
 		     {\
 			mutt_flushinp (); \
-			mutt_error _(Function_not_permitted_in_attach_message_mode); \
+			mutt_error ("Function not permitted in attach-message mode."); \
 			break; \
 		     }
 
@@ -304,13 +299,6 @@ int mutt_index_menu (void)
     menu->max = Context ? Context->vcount : 0;
     oldcount = Context ? Context->msgcount : 0;
 
-    /* check if we need to resort the index because just about
-     * any 'op' below could do mutt_enter_command(), either here or
-     * from any new menu launched, and change $sort/$sort_aux
-     */
-    if (option (OPTNEEDRESORT) && Context && Context->msgcount)
-      resort_index (menu);
-
     if (Context && !attach_msg)
     {
       int check;
@@ -792,7 +780,7 @@ int mutt_index_menu (void)
 	  /* calculate the number of messages _above_ the cursor,
 	   * so we can keep the cursor on the current message
 	   */ 
-	  for (j = 0; j <= menu->current; j++)
+	  for (j = 0; j < menu->current; j++)
 	  {
 	    if (Context->hdrs[Context->v2r[j]]->deleted)
 	      dcount++;
@@ -868,17 +856,17 @@ int mutt_index_menu (void)
 
       case OP_MAIN_CHANGE_FOLDER:
       
-	if (attach_msg)
+	if (attach_msg || option (OPTREADONLY))
 	  op = OP_MAIN_CHANGE_FOLDER_READONLY;
 
 	/* fallback to the readonly case */
 
       case OP_MAIN_CHANGE_FOLDER_READONLY:
 
-        if ((op == OP_MAIN_CHANGE_FOLDER_READONLY) || option (OPTREADONLY))
-          cp = _("Open mailbox in read-only mode");
-        else
+        if (op == OP_MAIN_CHANGE_FOLDER)
           cp = _("Open mailbox");
+        else
+          cp = _("Open mailbox in read-only mode");
 
 	buf[0] = '\0';
 	mutt_buffy (buf);
@@ -961,8 +949,13 @@ int mutt_index_menu (void)
 	  break;
 	}
 
+	if (option (OPTNEEDRESORT) && Context && Context->msgcount)
+	{
+	  resort_index (menu);
+	}
+
 	menu->menu = MENU_PAGER;
- 	menu->oldcurrent = menu->current;
+	menu->oldcurrent = menu->current;
 	continue;
 
       case OP_EXIT:
@@ -1144,7 +1137,7 @@ int mutt_index_menu (void)
 	    }
 	  }
 
-	  if ((Sort & SORT_MASK) == SORT_THREADS && CUR->collapsed)
+	  if (CUR->collapsed)
 	  {
 	    if ((op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD) &&
 		UNREAD (CUR))
@@ -1481,6 +1474,10 @@ int mutt_index_menu (void)
 	CurrentMenu = MENU_MAIN;
 	mutt_enter_command ();
 	mutt_check_rescore (Context);
+	if (option (OPTNEEDRESORT) && Context && Context->msgcount)
+	{
+	  resort_index (menu);
+	}
 	if (option (OPTFORCEREDRAWINDEX))
 	  menu->redraw = REDRAW_FULL;
 	unset_option (OPTFORCEREDRAWINDEX);
