@@ -1,4 +1,3 @@
-static const char rcsid[]="$Id: from.c,v 2.1 1998/12/21 10:37:32 roessler Exp $";
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
@@ -179,14 +178,12 @@ time_t is_from (const char *s, char *path, size_t pathlen)
 
   /* year */
   if (sscanf (s, "%d", &yr) != 1) return 0;
-  tm.tm_year = yr > 1900 ? yr - 1900 : yr;
-
+  tm.tm_year = yr > 1900 ? yr - 1900 : (yr < 70 ? yr + 100 : yr);
+  
   dprint (3,(debugfile, "is_from(): month=%d, day=%d, hr=%d, min=%d, sec=%d, yr=%d.\n",
 	     tm.tm_mon, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec, tm.tm_year));
 
-  tm.tm_isdst = 0;
-  tm.tm_yday = 0;
-  tm.tm_wday = 0;
+  tm.tm_isdst = -1;
 
   return (mutt_mktime (&tm, 0));
 }
