@@ -18,50 +18,10 @@
 
 #include "mutt.h"
 
-
-
 #ifdef _PGPPATH
 #include "pgp.h"
 #endif
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 #include <sys/stat.h>
 #include <string.h>
 #include <ctype.h>
@@ -90,7 +50,7 @@ void mutt_edit_headers (const char *editor,
     return;
   }
 
-  mutt_write_rfc822_header (ofp, msg->env, NULL, 1);
+  mutt_write_rfc822_header (ofp, msg->env, NULL, 1, 0);
   fputc ('\n', ofp);	/* tie off the header. */
 
   /* now copy the body of the message. */
@@ -141,7 +101,6 @@ void mutt_edit_headers (const char *editor,
   mutt_free_envelope (&msg->env);
   msg->env = n;
 
-  msg->env->from = mutt_expand_aliases (msg->env->from);
   msg->env->to = mutt_expand_aliases (msg->env->to);
   msg->env->cc = mutt_expand_aliases (msg->env->cc);
   msg->env->bcc = mutt_expand_aliases (msg->env->bcc);
