@@ -24,6 +24,7 @@
 #include "globals.h"
 #include "mutt_socket.h"
 #include "sort.h"
+#include "browser.h"
 
 #include <unistd.h>
 #include <netinet/in.h>
@@ -48,7 +49,7 @@
 
 
 
-/* Minimal support for IMAP 4rev1. Slightly more minimal for IMAP 4. */
+/* Minimal support for IMAP 4rev1 */
 
 #define IMAP_PORT 143
 
@@ -63,57 +64,7 @@ enum
   IMAP_NEW_MAIL,
   IMAP_EXPUNGE,
   IMAP_BYE,
-  IMAP_OK_FAIL,
-  IMAP_REOPENED
-};
-
-/* Capabilities */
-enum
-{
-  IMAP4 = 0,
-  IMAP4REV1,
-  STATUS,
-  ACL,				/* RFC 2086: IMAP4 ACL extension */
-  NAMESPACE,                   	/* RFC 2342: IMAP4 Namespace */
-  /* From here down, we don't care */
-  ACRAM_MD5,			/* AUTH=CRAM-MD5 */
-  AKERBEROS_V4,			/* AUTH=KERBEROS_V4 */
-  AGSSAPI,			/* AUTH=GSSAPI */
-  ALOGIN,			/* AUTH=LOGIN */
-  AUTH_LOGIN,			/* AUTH-LOGIN */
-  APLAIN,			/* AUTH=PLAIN */
-  ASKEY,			/* AUTH=SKEY */
-  IDLE,				/* RFC 2177: IMAP4 IDLE command */
-  LOGIN_REFERRALS,		/* LOGIN-REFERRALS */
-  MAILBOX_REFERRALS,		/* MAILBOX-REFERRALS */
-  SCAN,
-  SORT,
-  TORDEREDSUBJECT,		/* THREAD=ORDEREDSUBJECT */
-  UIDPLUS,			/* RFC 2859: IMAP4 UIDPLUS extension */
-  
-  CAPMAX
-};
-
-static char *Capabilities[] = {"IMAP4", "IMAP4rev1", "STATUS", "ACL", 
-  "NAMESPACE", "AUTH=CRAM-MD5", "AUTH=KERBEROS_V4", "AUTH=GSSAPI", 
-  "AUTH=LOGIN", "AUTH-LOGIN", "AUTH=PLAIN", "AUTH=SKEY", "IDLE", 
-  "LOGIN-REFERRALS", "MAILBOX-REFERRALS", "QUOTA", "SCAN", "SORT", 
-  "THREAD=ORDEREDSUBJECT", "UIDPLUS", NULL};
-
-/* ACL Rights */
-enum
-{
-  IMAP_ACL_LOOKUP = 0,
-  IMAP_ACL_READ,
-  IMAP_ACL_SEEN,
-  IMAP_ACL_WRITE,
-  IMAP_ACL_INSERT,
-  IMAP_ACL_POST,
-  IMAP_ACL_CREATE,
-  IMAP_ACL_DELETE,
-  IMAP_ACL_ADMIN,
-
-  RIGHTSMAX
+  IMAP_OK_FAIL
 };
 
 enum
@@ -136,15 +87,12 @@ typedef struct
   /* This data is specific to a CONNECTION to an IMAP server */
   short status;
   short state;
-  short check_status;
   char delim;
-  unsigned char capabilities[(CAPMAX + 7)/8];
   CONNECTION *conn;
 
   /* The following data is all specific to the currently SELECTED mbox */
   CONTEXT *selected_ctx;
   char *selected_mailbox;
-  unsigned char rights[(RIGHTSMAX + 7)/8];
   unsigned int newMailCount;
   IMAP_CACHE cache[IMAP_CACHE_LEN];
 } IMAP_DATA;
@@ -171,26 +119,6 @@ typedef struct imap_header_info
   struct imap_header_info *next;
 } IMAP_HEADER_INFO;
 
-enum
-{
-  /* Namespace types */
-  IMAP_NS_PERSONAL = 0,
-  IMAP_NS_OTHER,
-  IMAP_NS_SHARED
-};
-
-typedef struct 
-{
-  int type;
-  int listable;
-  char *prefix;
-  char delim;
-  int home_namespace;
-  /* We get these when we check if namespace exists - cache them */
-  int noselect;			
-  int noinferiors;
-} IMAP_NAMESPACE_INFO;
-
 static void imap_make_sequence (char *buf, size_t buflen)
 {
   static int sequence = 0;
@@ -462,44 +390,6 @@ static char *imap_next_word (char *s)
   return s;
 }
 
-/* a is a word, b a string of words */
-static int imap_wordcasecmp(const char *a, const char *b)
-{
-  char tmp[SHORT_STRING];
-  char *s = (char *)b;
-  int i;
-
-  tmp[SHORT_STRING-1] = 0;
-  for(i=0;i < SHORT_STRING-2;i++,s++)
-  {
-    if (!*s || ISSPACE(*s))
-    {
-      tmp[i] = 0;
-      break;
-    }
-    tmp[i] = *s;
-  }
-  tmp[i+1] = 0;
-  return mutt_strcasecmp(a, tmp);
-    
-}
-
-static void imap_parse_capabilities (IMAP_DATA *idata, char *s)
-{
-  int x;
-
-  while (*s) 
-  {
-    for (x = 0; x < CAPMAX; x++)
-      if (imap_wordcasecmp(Capabilities[x], s) == 0)
-      {
-	mutt_bit_set (idata->capabilities, x);
-	break;
-      }
-    s = imap_next_word (s);
-  }   
-}
-
 static int imap_handle_untagged (IMAP_DATA *idata, char *s)
 {
   char *pn;
@@ -543,50 +433,6 @@ static int imap_handle_untagged (IMAP_DATA *idata, char *s)
        idata->status = IMAP_EXPUNGE;
     }
   }
-  else if (mutt_strncasecmp ("CAPABILITY", s, 10) == 0)
-  {
-    /* parse capabilities */
-    imap_parse_capabilities (idata, s);
-  }
-  else if (mutt_strncasecmp ("MYRIGHTS", s, 8) == 0)
-  {
-    s = imap_next_word (s);
-    s = imap_next_word (s);
-    while (*s && !isspace(*s))
-    {
-      switch (*s) 
-      {
-	case 'l':
-	  mutt_bit_set (idata->rights, IMAP_ACL_LOOKUP);
-	  break;
-	case 'r':
-	  mutt_bit_set (idata->rights, IMAP_ACL_READ);
-	  break;
-	case 's':
-	  mutt_bit_set (idata->rights, IMAP_ACL_SEEN);
-	  break;
-	case 'w':
-	  mutt_bit_set (idata->rights, IMAP_ACL_WRITE);
-	  break;
-	case 'i':
-	  mutt_bit_set (idata->rights, IMAP_ACL_INSERT);
-	  break;
-	case 'p':
-	  mutt_bit_set (idata->rights, IMAP_ACL_POST);
-	  break;
-	case 'c':
-	  mutt_bit_set (idata->rights, IMAP_ACL_CREATE);
-	  break;
-	case 'd':
-	  mutt_bit_set (idata->rights, IMAP_ACL_DELETE);
-	  break;
-	case 'a':
-	  mutt_bit_set (idata->rights, IMAP_ACL_ADMIN);
-	  break;
-      }
-      s++;
-    }
-  }
   else if (mutt_strncasecmp ("BYE", s, 3) == 0)
   {
     /* server shut down our connection */
@@ -607,22 +453,6 @@ static int imap_handle_untagged (IMAP_DATA *idata, char *s)
   return 0;
 }
 
-static int get_literal_count(const char *buf, long *bytes)
-{
-  char *pc;
-  char *pn;
-
-  if (!(pc = strchr (buf, '{')))
-    return (-1);
-  pc++;
-  pn = pc;
-  while (isdigit (*pc))
-    pc++;
-  *pc = 0;
-  *bytes = atoi(pn);
-  return (0);
-}
-
 /*
  * Changed to read many headers instead of just one. It will return the
  * msgno of the last message read. It will return a value other than
@@ -631,18 +461,16 @@ static int get_literal_count(const char *buf, long *bytes)
 static int imap_read_headers (CONTEXT *ctx, int msgbegin, int msgend)
 {
   char buf[LONG_STRING],fetchbuf[LONG_STRING];
-  char hdrreq[STRING];
   FILE *fp;
   char tempfile[_POSIX_PATH_MAX];
   char seq[8];
   char *pc,*fpc,*hdr;
+  char *pn;
   long ploc;
   long bytes = 0;
   int msgno,fetchlast;
   IMAP_HEADER_INFO *h0,*h,*htemp;
-#define WANT_HEADERS  "DATE FROM SUBJECT TO CC MESSAGE-ID REFERENCES CONTENT-TYPE IN-REPLY-TO REPLY-TO" 
-  const char *want_headers = WANT_HEADERS;
-  int using_body_peek = 0;
+
   fetchlast = 0;
 
   /*
@@ -673,27 +501,7 @@ static int imap_read_headers (CONTEXT *ctx, int msgbegin, int msgend)
        * If we get more messages while doing this, we make another
        * request for all the new messages.
        */
-      if (mutt_bit_isset(CTX_DATA->capabilities,IMAP4REV1))
-      {
-	snprintf(hdrreq, sizeof (hdrreq), "BODY.PEEK[HEADER.FIELDS (%s)]", 
-		want_headers); 
-	using_body_peek = 1;
-      } 
-      else if (mutt_bit_isset(CTX_DATA->capabilities,IMAP4))
-      {
-	snprintf(hdrreq, sizeof (hdrreq), "RFC822.HEADER.LINES (%s)", 
-		want_headers);
-      }
-      else
-      {	/* Unable to fetch headers for lower versions */
-	mutt_error _("Unable to fetch headers from this IMAP server version.");
-	sleep (1);	/* pause a moment to let the user see the error */
-	return (-1);
-      }
-      snprintf (buf, sizeof (buf), 
-		"%s FETCH %d:%d (FLAGS INTERNALDATE RFC822.SIZE %s)\r\n", 
-		seq, msgno + 1, msgend + 1, hdrreq);
-
+      snprintf (buf, sizeof (buf), "%s FETCH %d:%d (FLAGS INTERNALDATE RFC822.SIZE BODY.PEEK[HEADER.FIELDS (DATE FROM SUBJECT TO CC MESSAGE-ID REFERENCES CONTENT-TYPE IN-REPLY-TO REPLY-TO)])\r\n", seq, msgno + 1, msgend + 1);
       mutt_socket_write (CTX_DATA->conn, buf);
       fetchlast = msgend + 1;
     }
@@ -723,25 +531,24 @@ static int imap_read_headers (CONTEXT *ctx, int msgbegin, int msgend)
           fpc=fetchbuf;
           while (*pc != '\0' && *pc != ')')
           {
-	    if (using_body_peek) 
-	      hdr=strstr(pc,"BODY");
-	    else
-	      hdr=strstr(pc,"RFC822.HEADER");
-	    if (!hdr)
-            {
-              imap_error ("imap_read_headers()", buf);
-              return (-1);
-            }
+            hdr=strstr(pc,"BODY");
             strncpy(fpc,pc,hdr-pc);
             fpc += hdr-pc;
             *fpc = '\0';
             pc=hdr;
             /* get some number of bytes */
-	    if (get_literal_count(buf, &bytes) < 0)
+            if (!(pc = strchr (pc, '{')))
             {
               imap_error ("imap_read_headers()", buf);
               return (-1);
             }
+            pc++;
+            pn = pc;
+            while (isdigit (*pc))
+              pc++;
+            *pc = 0;
+            bytes = atoi (pn);
+
             imap_read_bytes (fp, CTX_DATA->conn, bytes);
             if (mutt_socket_read_line_d (buf, sizeof (buf), CTX_DATA->conn) < 0)
             {
@@ -953,7 +760,7 @@ static int imap_reopen_mailbox (CONTEXT *ctx, int *index_hint)
       }
       if (!found)
       {
-	for (j = 0; j < i && j < old_msgcount; j++)
+	for (j = 0; j < i; j++)
 	{
 	  if (old_hdrs[j] == NULL)
 	    continue;
@@ -1049,12 +856,10 @@ static int imap_exec (char *buf, size_t buflen, IMAP_DATA *idata,
 
       count = imap_read_headers (idata->selected_ctx, 
 	  idata->selected_ctx->msgcount, count - 1) + 1;
-      idata->check_status = IMAP_NEW_MAIL;
     }
     else
     {
       imap_reopen_mailbox (idata->selected_ctx, NULL);
-      idata->check_status = IMAP_REOPENED;
     }
 
     idata->status = 0;
@@ -1198,45 +1003,6 @@ static char *imap_fix_path (IMAP_DATA *idata, char *mailbox, char *path,
   return path;
 }
 
-static int imap_check_acl (IMAP_DATA *idata)
-{
-  char buf[LONG_STRING];
-  char mbox[LONG_STRING];
-  char seq[16];
-
-  imap_make_sequence (seq, sizeof (seq));
-  imap_quote_string (mbox, sizeof(mbox), idata->selected_mailbox);
-  snprintf (buf, sizeof (buf), "%s MYRIGHTS %s\r\n", seq, mbox);
-  if (imap_exec (buf, sizeof (buf), idata, seq, buf, 0) != 0)
-  {
-    imap_error ("imap_check_acl", buf);
-    return (-1);
-  }
-  return (0);
-}
-
-static int imap_check_capabilities (IMAP_DATA *idata)
-{
-  char buf[LONG_STRING];
-  char seq[16];
-
-  imap_make_sequence (seq, sizeof (seq));
-  snprintf (buf, sizeof (buf), "%s CAPABILITY\r\n", seq);
-  if (imap_exec (buf, sizeof (buf), idata, seq, buf, 0) != 0)
-  {
-    imap_error ("imap_check_capabilities", buf);
-    return (-1);
-  }
-  if (!(mutt_bit_isset(idata->capabilities,IMAP4)
-      ||mutt_bit_isset(idata->capabilities,IMAP4REV1)))
-  {
-    mutt_error _("This IMAP server is ancient. Mutt does not work with it.");
-    sleep (5);	/* pause a moment to let the user see the error */
-    return (-1);
-  }
-  return (0);
-}
-
 static int imap_authenticate (IMAP_DATA *idata, CONNECTION *conn)
 {
   char buf[LONG_STRING];
@@ -1328,24 +1094,14 @@ static int imap_open_connection (IMAP_DATA *idata, CONNECTION *conn)
 
   if (mutt_strncmp ("* OK", buf, 4) == 0)
   {
-    if (imap_check_capabilities(idata) != 0 
-	|| imap_authenticate (idata, conn) != 0)
+    if (imap_authenticate (idata, conn) != 0)
     {
       close (conn->fd);
       idata->state = IMAP_DISCONNECTED;
       return (-1);
     }
   }
-  else if (mutt_strncmp ("* PREAUTH", buf, 9) == 0)
-  {
-    if (imap_check_capabilities(idata) != 0)
-    {
-      close (conn->fd);
-      idata->state = IMAP_DISCONNECTED;
-      return (-1);
-    }
-  } 
-  else
+  else if (mutt_strncmp ("* PREAUTH", buf, 9) != 0)
   {
     imap_error ("imap_open_connection()", buf);
     close (conn->fd);
@@ -1397,7 +1153,6 @@ int imap_open_mailbox (CONTEXT *ctx)
 
   /* Clean up path and replace the one in the ctx */
   imap_fix_path (idata, pc, buf, sizeof (buf));
-  FREE(&(idata->selected_mailbox));
   idata->selected_mailbox = safe_strdup (buf);
   if (port != IMAP_PORT)
     snprintf (buf, sizeof (buf), "{%s:%d}%s", host, port, 
@@ -1456,25 +1211,6 @@ int imap_open_mailbox (CONTEXT *ctx)
     return (-1);
   }
 
-  if (mutt_bit_isset (idata->capabilities, ACL))
-  {
-    if (imap_check_acl (idata))
-    {
-      return (-1);
-    }
-  }
-  else
-  {
-    mutt_bit_set (idata->rights, IMAP_ACL_LOOKUP);
-    mutt_bit_set (idata->rights, IMAP_ACL_READ);
-    mutt_bit_set (idata->rights, IMAP_ACL_SEEN);
-    mutt_bit_set (idata->rights, IMAP_ACL_WRITE);
-    mutt_bit_set (idata->rights, IMAP_ACL_INSERT);
-    mutt_bit_set (idata->rights, IMAP_ACL_POST);
-    mutt_bit_set (idata->rights, IMAP_ACL_CREATE);
-    mutt_bit_set (idata->rights, IMAP_ACL_DELETE);
-  }
-
   ctx->hdrmax = count;
   ctx->hdrs = safe_malloc (count * sizeof (HEADER *));
   ctx->v2r = safe_malloc (count * sizeof (int));
@@ -1542,27 +1278,8 @@ int imap_open_mailbox_append (CONTEXT *ctx)
 
   imap_quote_string (mbox, sizeof (mbox), mailbox);
   imap_make_sequence (seq, sizeof (seq));
-				
-  if (mutt_bit_isset(idata->capabilities,IMAP4REV1))
-  {
-    snprintf (buf, sizeof (buf), "%s STATUS %s (UIDVALIDITY)\r\n", seq, mbox);
-  }
-  else if (mutt_bit_isset(idata->capabilities,STATUS))
-  { 
-    /* We have no idea what the other guy wants. UW imapd 8.3 wants this
-     * (but it does not work if another mailbox is selected) */
-    snprintf (buf, sizeof (buf), "%s STATUS %s (UID-VALIDITY)\r\n", seq, mbox);
-  }
-  else
-  {
-    /* STATUS not supported
-     * The thing to do seems to be:
-     *  - Open a *new* IMAP session, select, and then close it. Report the
-     * error if the mailbox did not exist. */
-    mutt_message _("Unable to append to IMAP mailboxes at this server");
-    return (-1);
-  }
-
+  snprintf (buf, sizeof (buf), "%s STATUS %s (UIDVALIDITY)\r\n", seq, mbox);
+      
   r = imap_exec (buf, sizeof (buf), idata, seq, buf, IMAP_OK_FAIL);
   if (r == -2)
   {
@@ -1594,6 +1311,7 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
   char buf[LONG_STRING];
   char path[_POSIX_PATH_MAX];
   char *pc;
+  char *pn;
   long bytes;
   int pos, len;
   IMAP_CACHE *cache;
@@ -1638,7 +1356,7 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
   mutt_socket_write (CTX_DATA->conn, buf);
   do
   {
-    if (mutt_socket_read_line_d (buf, sizeof (buf), CTX_DATA->conn) < 0)
+    if (mutt_socket_read_line (buf, sizeof (buf), CTX_DATA->conn) < 0)
     {
       return (-1);
     }
@@ -1650,35 +1368,25 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
       pc = imap_next_word (pc);
       if (mutt_strncasecmp ("FETCH", pc, 5) == 0)
       {
-	while (*pc)
+	if (!(pc = strchr (buf, '{')))
 	{
-	  pc = imap_next_word (pc);
-	  if (pc[0] == '(')
-	    pc++;
-	  dprint (2, (debugfile, "Found FETCH word %s\n", pc));
-	  if (strncasecmp ("RFC822", pc, 6) == 0)
-	  {
-	    pc = imap_next_word (pc);
-	    if (get_literal_count(pc, &bytes) < 0)
-	    {
-	      imap_error ("imap_fetch_message()", buf);
-	      return (-1);
-	    }
-	    for (pos = 0; pos < bytes; )
-	    {
-	      len = mutt_socket_read_line (buf, sizeof (buf), CTX_DATA->conn);
-	      if (len < 0)
-		return (-1);
-	      pos += len;
-	      fputs (buf, msg->fp);
-	      fputs ("\n", msg->fp);
-	    }
-	    if (mutt_socket_read_line (buf, sizeof (buf), CTX_DATA->conn) < 0)
-	    {
-	      return (-1);
-	    }
-	    pc = buf;
-	  }
+	  imap_error ("imap_fetch_message()", buf);
+	  return (-1);
+	}
+	pc++;
+	pn = pc;
+	while (isdigit (*pc))
+	  pc++;
+	*pc = 0;
+	bytes = atoi (pn);
+	for (pos = 0; pos < bytes; )
+	{
+	  len = mutt_socket_read_line (buf, sizeof (buf), CTX_DATA->conn);
+	  if (len < 0)
+	    return (-1);
+	  pos += len;
+	  fputs (buf, msg->fp);
+	  fputs ("\n", msg->fp);
 	}
       }
       else if (imap_handle_untagged (CTX_DATA, buf) != 0)
@@ -1697,15 +1405,6 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
   rewind (msg->fp);
   mutt_free_envelope (&ctx->hdrs[msgno]->env);
   ctx->hdrs[msgno]->env = mutt_read_rfc822_header (msg->fp, ctx->hdrs[msgno],0);
-  fgets (buf, sizeof (buf), msg->fp);
-  while (!feof (msg->fp))
-  {
-    ctx->hdrs[msgno]->lines++;
-    fgets (buf, sizeof (buf), msg->fp);
-  }
-
-  ctx->hdrs[msgno]->content->length = ftell (msg->fp) - 
-                                        ctx->hdrs[msgno]->content->offset;
 
   /* This needs to be done in case this is a multipart message */
 #ifdef _PGPPATH
@@ -1872,24 +1571,11 @@ int imap_close_connection (CONTEXT *ctx)
   return 0;
 }
 
-static void _imap_set_flag (CONTEXT *ctx, int aclbit, int flag, const char *str, 
-		     char *sf, char *uf)
-{
-  if (mutt_bit_isset (CTX_DATA->rights, aclbit))
-  {
-    if (flag)
-      strcat (sf, str);
-    else
-      strcat (uf, str);
-  }
-}
-
 int imap_sync_mailbox (CONTEXT *ctx)
 {
   char seq[8];
   char buf[LONG_STRING];
-  char set_flags[LONG_STRING];
-  char unset_flags[LONG_STRING];
+  char tmp[LONG_STRING];
   int n;
 
   /* save status changes */
@@ -1897,60 +1583,42 @@ int imap_sync_mailbox (CONTEXT *ctx)
   {
     if (ctx->hdrs[n]->deleted || ctx->hdrs[n]->changed)
     {
-      snprintf (buf, sizeof (buf), _("Saving message status flags... [%d/%d]"),
-		n+1, ctx->msgcount);
-      mutt_message (buf);
-      
-      *set_flags = '\0';
-      *unset_flags = '\0';
-      
-      _imap_set_flag (ctx, IMAP_ACL_SEEN, ctx->hdrs[n]->read, "\\Seen ", set_flags, unset_flags);
-      _imap_set_flag (ctx, IMAP_ACL_WRITE, ctx->hdrs[n]->flagged, "\\Flagged ", set_flags, unset_flags);
-      _imap_set_flag (ctx, IMAP_ACL_WRITE, ctx->hdrs[n]->replied, "\\Answered ", set_flags, unset_flags);
-      _imap_set_flag (ctx, IMAP_ACL_DELETE, ctx->hdrs[n]->deleted, "\\Deleted", set_flags, unset_flags);
-      
-      mutt_remove_trailing_ws (set_flags);
-      mutt_remove_trailing_ws (unset_flags);
-      
-      if (*set_flags)
+      snprintf (tmp, sizeof (tmp), _("Saving message status flags... [%d/%d]"),
+	n+1, ctx->msgcount);
+      mutt_message (tmp);
+      *tmp = 0;
+      if (ctx->hdrs[n]->read)
+	strcat (tmp, "\\Seen ");
+      if (ctx->hdrs[n]->flagged)
+	strcat (tmp, "\\Flagged ");
+      if (ctx->hdrs[n]->replied)
+	strcat (tmp, "\\Answered ");
+      if (ctx->hdrs[n]->deleted)
+        strcat (tmp, "\\Deleted");
+      mutt_remove_trailing_ws (tmp);
+
+      if (!*tmp) continue; /* imapd doesn't like empty flags. */
+      imap_make_sequence (seq, sizeof (seq));
+      snprintf (buf, sizeof (buf), "%s STORE %d FLAGS.SILENT (%s)\r\n", seq, 
+      	ctx->hdrs[n]->index + 1, tmp);
+      if (imap_exec (buf, sizeof (buf), CTX_DATA, seq, buf, 0) != 0)
       {
-	imap_make_sequence (seq, sizeof (seq));
-	snprintf (buf, sizeof (buf), "%s STORE %d +FLAGS.SILENT (%s)\r\n", seq,
-		  ctx->hdrs[n]->index + 1, set_flags);
-	if (imap_exec (buf, sizeof (buf), CTX_DATA, seq, buf, 0) != 0)
-	{
-	  imap_error ("imap_sync_mailbox()", buf);
-	  return (-1);
-	}
-      }
-      
-      if (*unset_flags)
-      {
-	imap_make_sequence (seq, sizeof (seq));
-	snprintf (buf, sizeof (buf), "%s STORE %d -FLAGS.SILENT (%s)\r\n", seq,
-		  ctx->hdrs[n]->index + 1, unset_flags);
-	if (imap_exec (buf, sizeof (buf), CTX_DATA, seq, buf, 0) != 0)
-	{
-	  imap_error ("imap_sync_mailbox()", buf);
-	  return (-1);
-	}
+	imap_error ("imap_sync_mailbox()", buf);
+	return (-1);
       }
     }
   }
 
-  if (mutt_bit_isset(CTX_DATA->rights, IMAP_ACL_DELETE))
+  mutt_message _("Expunging messages from server...");
+  CTX_DATA->status = IMAP_EXPUNGE;
+  imap_make_sequence (seq, sizeof (seq));
+  snprintf (buf, sizeof (buf), "%s EXPUNGE\r\n", seq);
+  if (imap_exec (buf, sizeof (buf), CTX_DATA, seq, buf, 0) != 0)
   {
-    mutt_message _("Expunging messages from server...");
-    CTX_DATA->status = IMAP_EXPUNGE;
-    imap_make_sequence (seq, sizeof (seq));
-    snprintf (buf, sizeof (buf), "%s EXPUNGE\r\n", seq);
-    if (imap_exec (buf, sizeof (buf), CTX_DATA, seq, buf, 0) != 0)
-    {
-      imap_error ("imap_sync_mailbox()", buf);
-      return (-1);
-    }
-    CTX_DATA->status = 0;
+    imap_error ("imap_sync_mailbox()", buf);
+    return (-1);
   }
+  CTX_DATA->status = 0;
 
   for (n = 0; n < IMAP_CACHE_LEN; n++)
   {
@@ -2012,14 +1680,7 @@ void imap_fastclose_mailbox (CONTEXT *ctx)
   }
 }
 
-/* use the NOOP command to poll for new mail
- *
- * return values:
- *	M_REOPENED	mailbox has been reopened
- *	M_NEW_MAIL	new mail has arrived!
- *	0		no change
- *	-1		error
- */
+/* use the NOOP command to poll for new mail */
 int imap_check_mailbox (CONTEXT *ctx, int *index_hint)
 {
   char seq[8];
@@ -2034,7 +1695,6 @@ int imap_check_mailbox (CONTEXT *ctx, int *index_hint)
     checktime=k;
   }
 
-  CTX_DATA->check_status = 0;
   imap_make_sequence (seq, sizeof (seq));
   snprintf (buf, sizeof (buf), "%s NOOP\r\n", seq);
   if (imap_exec (buf, sizeof (buf), CTX_DATA, seq, buf, 0) != 0)
@@ -2043,11 +1703,7 @@ int imap_check_mailbox (CONTEXT *ctx, int *index_hint)
     return (-1);
   }
 
-  if (CTX_DATA->check_status == IMAP_NEW_MAIL)
-    return M_NEW_MAIL;
-  if (CTX_DATA->check_status == IMAP_REOPENED)
-    return M_REOPENED;
-  return 0;
+  return (msgcount != ctx->msgcount);
 }
 
 int imap_buffy_check (char *path)
@@ -2057,7 +1713,6 @@ int imap_buffy_check (char *path)
   char host[SHORT_STRING];
   char buf[LONG_STRING];
   char mbox[LONG_STRING];
-  char mbox_unquoted[LONG_STRING];
   char seq[8];
   char *s;
   char *pc;
@@ -2092,31 +1747,9 @@ int imap_buffy_check (char *path)
   if (strlen (buf) < strlen (pc))
       strcpy (pc, buf);
 
-  imap_make_sequence (seq, sizeof (seq));		
+  imap_make_sequence (seq, sizeof (seq));
   imap_quote_string (mbox, sizeof(mbox), buf);
-  strfcpy (mbox_unquoted, buf, sizeof (mbox_unquoted));
-
-  /* The draft IMAP implementor's guide warns againts using the STATUS
-   * command on a mailbox that you have selected 
-   */
-
-  if (mutt_strcmp(mbox, idata->selected_mailbox) == 0
-      || (mutt_strcasecmp(mbox, "INBOX") == 0
-	  && mutt_strcasecmp(mbox, idata->selected_mailbox) == 0))
-  {
-    snprintf (buf, sizeof (buf), "%s NOOP\r\n", seq);
-  }
-  else if (mutt_bit_isset(idata->capabilities,IMAP4REV1) ||
-	   mutt_bit_isset(idata->capabilities,STATUS))
-  {				
-    snprintf (buf, sizeof (buf), "%s STATUS %s (RECENT)\r\n", seq, mbox);
-  }
-  else
-  {
-    /* Server does not support STATUS, and this is not the current mailbox.
-     * There is no lightweight way to check recent arrivals */
-      return (-1);
-  }
+  snprintf (buf, sizeof (buf), "%s STATUS %s (RECENT)\r\n", seq, mbox);
 
   mutt_socket_write (conn, buf);
 
@@ -2133,7 +1766,7 @@ int imap_buffy_check (char *path)
       if (mutt_strncasecmp ("STATUS", s, 6) == 0)
       {
 	s = imap_next_word (s);
-	if (mutt_strncmp (mbox_unquoted, s, mutt_strlen (mbox_unquoted)) == 0)
+	if (mutt_strncmp (mbox, s, mutt_strlen (mbox)) == 0)
 	{
 	  s = imap_next_word (s);
 	  s = imap_next_word (s);
@@ -2162,3 +1795,331 @@ int imap_buffy_check (char *path)
   return recent;
 }
 
+static void add_folder (char delim, char *folder, char *host, int port, 
+    int noselect, int noinferiors, struct browser_state *state)
+{
+  char tmp[LONG_STRING];
+  char idir[LONG_STRING];
+  int flen = strlen (folder);
+  int x;
+  char *f;
+
+  imap_unquote_string (folder);
+  for (x = 0; x < state->entrylen; x++)
+  {
+    f = strchr ((state->entry)[x].name, '}');
+    if (f)
+    {
+      int len = strlen (folder);
+
+      f++;
+      if (!strncmp (f, folder, len) && (f[len] == '\0' || f[len] == delim))
+	return;
+    }
+  }
+  if (state->entrylen + 1 == state->entrymax)
+  {
+    safe_realloc ((void **) &state->entry,
+	sizeof (struct folder_file) * (state->entrymax += 256));
+  }
+  if (!noselect)
+  {
+    if (port == IMAP_PORT)
+      snprintf (tmp, sizeof (tmp), "{%s}%s", host, folder);
+    else
+      snprintf (tmp, sizeof (tmp), "{%s:%d}%s", host, port, folder);
+    (state->entry)[state->entrylen].name = safe_strdup (tmp);
+    snprintf (tmp, sizeof (tmp), "IMAP %-25s %-25s", host, folder);
+    (state->entry)[state->entrylen].desc = safe_strdup (tmp);
+    (state->entry)[state->entrylen].notfolder = 0;
+    (state->entrylen)++;
+  }
+  if (!noinferiors)
+  {
+    int nodelim = 0;
+
+    if (delim == folder[flen - 1])
+      nodelim = 1;
+    if (port == IMAP_PORT)
+      snprintf (tmp, sizeof (tmp), "{%s}%s%c", host, folder, 
+	  nodelim ? '\0' : delim);
+    else
+      snprintf (tmp, sizeof (tmp), "{%s:%d}%s%c", host, port, folder, 
+	  nodelim ? '\0' : delim);
+    (state->entry)[state->entrylen].name = safe_strdup (tmp);
+    snprintf (idir, sizeof (idir), "%s%c", folder, 
+	nodelim ? '\0' : delim);
+    snprintf (tmp, sizeof (tmp), "IMAP %-25s %-25s", host, idir);
+    (state->entry)[state->entrylen].desc = safe_strdup (tmp);
+    (state->entry)[state->entrylen].notfolder = 1;
+    (state->entrylen)++;
+  }
+}
+
+static int add_namespace (IMAP_DATA *idata, char *host, int port, 
+    struct browser_state *state)
+{
+  char buf[LONG_STRING];
+  char seq[16];
+  char *s;
+  int n;
+  char ns[LONG_STRING];
+  char delim = '/';
+
+  imap_make_sequence (seq, sizeof (seq));
+  snprintf (buf, sizeof (buf), "%s NAMESPACE\r\n", seq);
+
+  mutt_socket_write (idata->conn, buf);
+  do 
+  {
+    if (mutt_socket_read_line_d (buf, sizeof (buf), idata->conn) < 0)
+    {
+      return (-1);
+    }
+
+    if (buf[0] == '*') 
+    {
+      s = imap_next_word (buf);
+      if (mutt_strncasecmp ("NAMESPACE", s, 9) == 0)
+      {
+	/* There are three sections to the response, User, Other, Shared,
+	 * and maybe more by extension */
+	while (*s)
+	{
+	  s = imap_next_word (s);
+	  if (*s && strncmp (s, "NIL", 3))
+	  {
+	    s++;
+	    while (*s && *s != ')')
+	    {
+	      s++; /* skip ( */
+	      /* copy namespace */
+	      n = 0;
+	      if (*s == '\"')
+	      {
+		s++;
+		while (*s && *s != '\"') 
+		{
+		  if (*s == '\\')
+		    s++;
+		  ns[n++] = *s;
+		  s++;
+		}
+	      }
+	      else
+		while (*s && !ISSPACE (*s)) 
+		{
+		  ns[n++] = *s;
+		  s++;
+		}
+	      ns[n] = '\0';
+	      /* delim? */
+	      s = imap_next_word (s);
+	      if (*s && *s == '\"')
+		if (s[1] && s[2] == '\"')
+		  delim = s[1];
+		else if (s[1] && s[1] == '\\' && s[2] && s[3] == '\"')
+		  delim = s[2];
+	      if (strcmp (ns, "INBOX"))
+		add_folder (delim, ns, host, port, 1, 0, state);
+	      while (*s && *s != ')') s++;
+	      s++;
+	    }
+	  }
+	}
+      }
+      else
+      {
+	if (imap_handle_untagged (idata, buf) != 0)
+	  return (-1);
+      }
+    }
+  }
+  while ((mutt_strncmp (buf, seq, SEQLEN) != 0));
+  return 0;
+}
+
+int imap_init_browse (char *path, struct browser_state *state)
+{
+  CONNECTION *conn;
+  IMAP_DATA *idata;
+  char buf[LONG_STRING];
+  char mbox[LONG_STRING];
+  char host[SHORT_STRING];
+  char seq[16];
+  char *ipath = NULL;
+  int n;
+  int port;
+  char *s;
+
+  if (imap_parse_path (path, host, sizeof (host), &port, &ipath))
+    return (-1);
+
+  conn = mutt_socket_select_connection (host, port, 0);
+  idata = CONN_DATA;
+
+  if (!idata || (idata->state == IMAP_DISCONNECTED))
+  {
+    if (!idata)
+    {
+      /* The current connection is a new connection */
+      idata = safe_calloc (1, sizeof (IMAP_DATA));
+      conn->data = idata;
+      idata->conn = conn;
+    }
+    if (imap_open_connection (idata, conn))
+      return (-1);
+  }
+
+  imap_fix_path (idata, ipath, mbox, sizeof (mbox));
+  for (n = mutt_strlen (mbox) - 1; n > 0 && mbox[n] != idata->delim ; n--);
+  if (n > 0)
+  {
+    int i;
+    for (i = n - 1; i > 0 && mbox[i] != idata->delim ; i--);
+    if (i > 0)
+    {
+      mbox[i] = '\0';
+      add_folder (idata->delim, mbox, host, port, 1, 0, state);
+      mbox[i] = idata->delim;
+    }
+    else if (i != 0)
+      add_folder (idata->delim, "INBOX", host, port, 0, 0, state);
+    else
+      add_folder (idata->delim, "/", host, port, 1, 0, state);
+
+    mbox[n] = '\0';
+    /* Don't duplicate INBOX here */
+    if (strcmp ("INBOX", mbox))
+      add_folder (idata->delim, mbox, host, port, 0, 1, state);
+    mbox[n] = idata->delim;
+    mbox[n+1] = '\0';
+  }
+  else
+  {
+    if (mbox[0] != '#')
+    {
+      if (idata->delim == '/')
+	strcpy (mbox, "/");
+      else if (idata->delim == '\\')
+	strcpy (mbox, "\\");
+      else
+	mbox[0] = '\0';
+      if (add_namespace (idata, host, port, state) < 0)
+	return (-1);
+      add_folder (idata->delim, "INBOX", host, port, 0, 0, state);
+    }
+  }
+  imap_make_sequence (seq, sizeof (seq));
+
+  mutt_message _("Contacted server, getting folder list...");
+  snprintf (buf, sizeof (buf), "%s %s \"\" \"%s%%\"\r\n", seq, 
+      option (OPTIMAPLSUB) ? "LSUB" : "LIST", mbox);
+
+  mutt_socket_write (conn, buf);
+
+  do 
+  {
+    if (mutt_socket_read_line_d (buf, sizeof (buf), conn) < 0)
+    {
+      return (-1);
+    }
+
+    if (buf[0] == '*') 
+    {
+      s = imap_next_word (buf);
+      if ((mutt_strncasecmp ("LIST", s, 4) == 0) ||
+	  (mutt_strncasecmp ("LSUB", s, 4) == 0))
+      {
+	int noselect = 0;
+	int noinferiors = 0;
+
+	s = imap_next_word (s); /* flags */
+	if (*s == '(')
+	{
+	  char *ep;
+
+	  s++;
+	  ep = s;
+	  while (*ep && *ep != ')') ep++;
+	  do {
+	    if (!strncmp (s, "\\NoSelect", 9))
+	      noselect = 1;
+	    if (!strncmp (s, "\\NoInferiors", 12))
+	      noinferiors = 1;
+	    if (*s != ')')
+	      s++;
+	    while (*s && *s != '\\' && *s != ')') s++;
+	  } while (s != ep);
+	}
+	else
+	  break;
+	s = imap_next_word (s); /* delim */
+	/* Reset the delimiter, this can change */
+	if (strncmp (s, "NIL", 3))
+	if (s && s[0] == '\"' && s[1] && s[2] == '\"')
+	  idata->delim = s[1];
+	else if (s && s[0] == '\"' && s[1] && s[1] == '\\' && s[2] && s[3] == '\"')
+	  idata->delim = s[2];
+	s = imap_next_word (s); /* name */
+	add_folder (idata->delim, s, host, port, noselect, noinferiors, state);
+      }
+      else
+      {
+	if (imap_handle_untagged (idata, buf) != 0)
+	  return (-1);
+      }
+    }
+  }
+  while ((mutt_strncmp (buf, seq, SEQLEN) != 0));
+  mutt_clear_error ();
+  return 0;
+}
+
+int imap_subscribe (char *path, int subscribe)
+{
+  CONNECTION *conn;
+  IMAP_DATA *idata;
+  char buf[LONG_STRING];
+  char mbox[LONG_STRING];
+  char host[SHORT_STRING];
+  char seq[16];
+  char *ipath = NULL;
+  int port;
+
+  if (imap_parse_path (path, host, sizeof (host), &port, &ipath))
+    return (-1);
+
+  conn = mutt_socket_select_connection (host, port, 0);
+  idata = CONN_DATA;
+
+  if (!idata || (idata->state == IMAP_DISCONNECTED))
+  {
+    if (!idata)
+    {
+      /* The current connection is a new connection */
+      idata = safe_calloc (1, sizeof (IMAP_DATA));
+      conn->data = idata;
+      idata->conn = conn;
+    }
+    if (imap_open_connection (idata, conn))
+      return (-1);
+  }
+
+  imap_fix_path (idata, ipath, buf, sizeof (buf));
+  if (subscribe)
+    mutt_message (_("Subscribing to %s..."), buf);
+  else
+    mutt_message (_("Unsubscribing to %s..."), buf);
+  imap_quote_string (mbox, sizeof(mbox), buf);
+  imap_make_sequence (seq, sizeof (seq));
+
+  snprintf (buf, sizeof (buf), "%s %s %s\r\n", seq, 
+      subscribe ? "SUBSCRIBE" : "UNSUBSCRIBE", mbox);
+
+  if (imap_exec (buf, sizeof (buf), idata, seq, buf, 0) < 0)
+  {
+    return (-1);
+  }
+  return 0;
+}
