@@ -70,14 +70,14 @@ struct option_t MuttVars[] = {
   { "abort_nosubject",	DT_QUAD, R_NONE, OPT_SUBJECT, M_ASKYES },
   { "abort_unmodified",	DT_QUAD, R_NONE, OPT_ABORT, M_YES },
   { "alias_file",	DT_PATH, R_NONE, UL &AliasFile, UL "~/.muttrc" },
-  { "alias_format",	DT_STR,  R_NONE, UL &AliasFmt, UL "%2n %t %-10a   %r" },
+  { "alias_format",	DT_STR,  R_NONE, UL &AliasFmt, UL "%4n %t %-10a   %r" },
   { "allow_8bit",	DT_BOOL, R_NONE, OPTALLOW8BIT, 1 },
   { "alternates",	DT_RX,	 R_BOTH, UL &Alternates, 0 },
   { "arrow_cursor",	DT_BOOL, R_BOTH, OPTARROWCURSOR, 0 },
   { "ascii_chars",	DT_BOOL, R_BOTH, OPTASCIICHARS, 0 },
   { "askbcc",		DT_BOOL, R_NONE, OPTASKBCC, 0 },
   { "askcc",		DT_BOOL, R_NONE, OPTASKCC, 0 },
-  { "attach_format",	DT_STR,  R_NONE, UL &AttachFormat, UL "%u%D%t%2n %T%.40d%> [%.7m/%.10M, %.6e, %s] " },
+  { "attach_format",	DT_STR,  R_NONE, UL &AttachFormat, UL "%u%D%t%4n %T%.40d%> [%.7m/%.10M, %.6e%?C?, %C?, %s] " },
   { "attach_split",	DT_BOOL, R_NONE, OPTATTACHSPLIT, 1 },
   { "attach_sep",	DT_STR,	 R_NONE, UL &AttachSep, UL "\n" },
   { "attribution",	DT_STR,	 R_NONE, UL &Attribution, UL "On %d, %n wrote:" },
@@ -124,11 +124,13 @@ struct option_t MuttVars[] = {
   { "hostname",		DT_STR,	 R_NONE, UL &Fqdn, 0 },
 #ifdef USE_IMAP
   { "imap_checkinterval", 	DT_NUM,	 R_NONE, UL &ImapCheckTime, 0 },
+  { "imap_list_subscribed",	DT_BOOL, R_NONE, OPTIMAPLSUB, 0 },
   { "imap_user",	DT_STR,  R_NONE, UL &ImapUser, UL 0 },
   { "imap_pass", 	DT_STR,  R_NONE, UL &ImapPass, UL 0 },
-  { "imap_passive",	DT_BOOL, R_NONE, OPTIMAPPASSIVE, 1 },
+  { "imap_passive",		DT_BOOL, R_NONE, OPTIMAPPASSIVE, 1 },
 #endif
   { "implicit_autoview", DT_BOOL,R_NONE, OPTIMPLICITAUTOVIEW, 0},
+  { "in_reply_to",	DT_STR,	 R_NONE, UL &InReplyTo, UL "%i; from %n on %{!%a, %b %d, %Y at %I:%M:%S%p %Z}" },
   { "include",		DT_QUAD, R_NONE, OPT_INCLUDE, M_ASKYES },
   { "indent_string",	DT_STR,	 R_NONE, UL &Prefix, UL "> " },
   { "indent_str",	DT_SYN,  R_NONE, UL "indent_string", 0 },
@@ -138,7 +140,7 @@ struct option_t MuttVars[] = {
   { "locale",		DT_STR,  R_BOTH, UL &Locale, UL "C" },
   { "mail_check",	DT_NUM,  R_NONE, UL &BuffyTimeout, 5 },
   { "mailcap_path",	DT_STR,	 R_NONE, UL &MailcapPath, 0 },
-  { "mailcap_sanitize", DT_BOOL, R_NONE, OPTMAILCAPSANITIZE, 1 },
+  { "mailcap_sanitize",	DT_BOOL, R_NONE, OPTMAILCAPSANITIZE, 1 },
   { "mark_old",		DT_BOOL, R_BOTH, OPTMARKOLD, 1 },
   { "markers",		DT_BOOL, R_PAGER, OPTMARKERS, 1 },
   { "mask",		DT_RX,	 R_NONE, UL &Mask, UL "!^\\.[^.]" },
@@ -151,6 +153,14 @@ struct option_t MuttVars[] = {
   { "mime_forward",	DT_QUAD, R_NONE, OPT_MIMEFWD, M_NO },
   { "mime_forward_decode", DT_BOOL, R_NONE, OPTMIMEFORWDECODE, 0 },
   { "mime_fwd",		DT_SYN,  R_NONE, UL "mime_forward", 0 },
+
+
+#ifdef MIXMASTER
+  { "mix_entry_format", DT_STR,  R_NONE, UL &MixEntryFormat, UL "%4n %c %-16s %a" },
+  { "mixmaster",	DT_PATH, R_NONE, UL &Mixmaster, UL MIXMASTER },
+#endif
+
+
   { "move",		DT_QUAD, R_NONE, OPT_MOVE, M_ASKNO },
   { "message_format",	DT_STR,	 R_NONE, UL &MsgFmt, UL "%s" },
   { "msg_format",	DT_SYN,  R_NONE, UL "message_format", 0 },
@@ -167,9 +177,11 @@ struct option_t MuttVars[] = {
   { "pgp_autosign",	DT_BOOL, R_NONE, OPTPGPAUTOSIGN, 0 },
   { "pgp_autoencrypt",	DT_BOOL, R_NONE, OPTPGPAUTOENCRYPT, 0 },
   { "pgp_encryptself",	DT_BOOL, R_NONE, OPTPGPENCRYPTSELF, 1 },
+  { "pgp_entry_format", DT_STR,  R_NONE, UL &PgpEntryFormat, UL "%4n %t%f %4l/0x%k %-4a %2c %u" },
   { "pgp_long_ids",	DT_BOOL, R_NONE, OPTPGPLONGIDS, 0 },
   { "pgp_replyencrypt",	DT_BOOL, R_NONE, OPTPGPREPLYENCRYPT, 0 },
   { "pgp_replysign",	DT_BOOL, R_NONE, OPTPGPREPLYSIGN, 0 },
+  { "pgp_show_unusable", DT_BOOL, R_NONE, OPTPGPSHOWUNUSABLE, 1 },
   { "pgp_sign_as",	DT_STR,	 R_NONE, UL &PgpSignAs, 0 },
   { "pgp_sign_micalg",	DT_STR,	 R_NONE, UL &PgpSignMicalg, UL "pgp-md5" },
   { "pgp_strict_enc",	DT_BOOL, R_NONE, OPTPGPSTRICTENC, 1 },
@@ -186,11 +198,6 @@ struct option_t MuttVars[] = {
   { "pgp_v5_pubring",	DT_PATH, R_NONE, UL &PgpV3Pubring, 0 },
   { "pgp_v5_secring",	DT_PATH, R_NONE, UL &PgpV3Secring, 0 },
 
-  { "pgp_v6",		DT_PATH, R_NONE, UL &PgpV6, 0 },
-  { "pgp_v6_language",	DT_STR,	 R_NONE, UL &PgpV6Language, 0 },
-  { "pgp_v6_pubring",	DT_PATH, R_NONE, UL &PgpV6Pubring, 0 },
-  { "pgp_v6_secring",	DT_PATH, R_NONE, UL &PgpV6Secring, 0 },
-
   { "pgp_gpg",		DT_PATH, R_NONE, UL &PgpGpg, 0 },
   
 # ifdef HAVE_PGP2
@@ -248,6 +255,7 @@ struct option_t MuttVars[] = {
   { "save_address",	DT_BOOL, R_NONE, OPTSAVEADDRESS, 0 },
   { "save_empty",	DT_BOOL, R_NONE, OPTSAVEEMPTY, 1 },
   { "save_name",	DT_BOOL, R_NONE, OPTSAVENAME, 0 },
+  { "send_charset",	DT_STR,  R_NONE, UL &SendCharset, UL "" },
   { "sendmail",		DT_PATH, R_NONE, UL &Sendmail, UL SENDMAIL " -oem -oi" },
   { "sendmail_wait",	DT_NUM,  R_NONE, UL &SendmailWait, 0 },
   { "shell",		DT_PATH, R_NONE, UL &Shell, 0 },
@@ -337,6 +345,7 @@ struct command_t Commands[] = {
   { "auto_view",	parse_list,		UL &AutoViewList },
   { "alternative_order",	parse_list,	UL &AlternativeOrderList},
   { "bind",		mutt_parse_bind,	0 },
+  { "charset-hook",	mutt_parse_hook,	M_CHARSETHOOK },
 #ifdef HAVE_COLOR
   { "color",		mutt_parse_color,	0 },
   { "uncolor",		mutt_parse_uncolor,	0 },
