@@ -102,8 +102,15 @@ void menu_redraw_full (MUTTMENU *menu)
 {
   SETCOLOR (MT_COLOR_NORMAL);
   /* clear() doesn't optimize screen redraws */
+#if 0
   move (0, 0);
   clrtobot ();
+#else
+  /* my ncurses library seems to run into
+   * endless loops in clrtobot...
+   */
+  clear();
+#endif
 
   if (option (OPTHELP))
   {
@@ -274,19 +281,13 @@ void menu_redraw_current (MUTTMENU *menu)
 
 void menu_check_recenter (MUTTMENU *menu)
 {
-  if (menu->max <= menu->pagelen && menu->top != 0)
-  {
-    menu->top = 0;
-    set_option (OPTNEEDREDRAW);
-    menu->redraw |= REDRAW_INDEX;
-  }
-  else if (menu->current >= menu->top + menu->pagelen)
+  if (menu->current >= menu->top + menu->pagelen)
   {
     if (option (OPTMENUSCROLL))
       menu->top = menu->current - menu->pagelen + 1;
     else
       menu->top += menu->pagelen * ((menu->current - menu->top) / menu->pagelen);
-    menu->redraw |= REDRAW_INDEX;
+    menu->redraw = REDRAW_INDEX;
   }
   else if (menu->current < menu->top)
   {
@@ -298,7 +299,7 @@ void menu_check_recenter (MUTTMENU *menu)
       if (menu->top < 0)
 	menu->top = 0;
     }
-    menu->redraw |= REDRAW_INDEX;
+    menu->redraw = REDRAW_INDEX;
   }
 }
 
@@ -566,6 +567,14 @@ static int default_color (int i)
    return ColorDefs[MT_COLOR_NORMAL];
 }
 
+static int menu_search_generic (MUTTMENU *m, regex_t *re, int n)
+{
+  char buf[LONG_STRING];
+
+  m->make_entry (buf, sizeof (buf), m, n);
+  return (regexec (re, buf, 0, NULL, 0));
+}
+
 MUTTMENU *mutt_new_menu (void)
 {
   MUTTMENU *p = (MUTTMENU *) safe_calloc (1, sizeof (MUTTMENU));
@@ -576,6 +585,7 @@ MUTTMENU *mutt_new_menu (void)
   p->redraw = REDRAW_FULL;
   p->pagelen = PAGELEN;
   p->color = default_color;
+  p->search = menu_search_generic;
   return (p);
 }
 
@@ -703,11 +713,11 @@ int mutt_menuLoop (MUTTMENU *menu)
     mutt_curs_set (1);
 
 #if defined (USE_SLANG_CURSES) || defined (HAVE_RESIZETERM)
-    if (Signals & S_SIGWINCH)
+    if (SigWinch)
     {
       mutt_resize_screen ();
       menu->redraw = REDRAW_FULL;
-      Signals &= ~S_SIGWINCH;
+      SigWinch = 0;
     }
 #endif
 
@@ -802,11 +812,9 @@ int mutt_menuLoop (MUTTMENU *menu)
 	{
 	  if (menu->max)
 	  {
-	    if (menu->tag (menu, menu->current))
-	      menu->tagged++;
-	    else
-	      menu->tagged--;
-	    if (option (OPTRESOLVE) && menu->current < menu->max - 1)
+	    short i = menu->tag (menu, menu->current);
+	    menu->tagged += i;
+	    if (i && option (OPTRESOLVE) && menu->current < menu->max - 1)
 	    {
 	      menu->current++;
 	      menu->redraw = REDRAW_MOTION_RESYNCH;
