@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1999 Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -65,31 +66,28 @@
 
 #ifdef DL_STANDALONE
 
-static int invoke_dotlock(const char *path, int flags, int retry)
+static int invoke_dotlock (const char *path, int flags, int retry)
 {
   char cmd[LONG_STRING + _POSIX_PATH_MAX];
+  char f[SHORT_STRING + _POSIX_PATH_MAX];
   char r[SHORT_STRING];
-  char *f;
   
-  if(flags & DL_FL_RETRY)
-    snprintf(r, sizeof(r), "-r %d ", retry ? MAXLOCKATTEMPT : 0);
+  if (flags & DL_FL_RETRY)
+    snprintf (r, sizeof (r), "-r %d ", retry ? MAXLOCKATTEMPT : 0);
   
-  f = mutt_quote_filename(path);
+  mutt_quote_filename (f, sizeof (f), path);
   
-  snprintf(cmd, sizeof(cmd),
-	   "%s %s%s%s%s%s%s%s",
+  snprintf (cmd, sizeof (cmd),
+	   "%s %s%s%s%s%s%s",
 	   DOTLOCK,
 	   flags & DL_FL_TRY ? "-t " : "",
 	   flags & DL_FL_UNLOCK ? "-u " : "",
 	   flags & DL_FL_USEPRIV ? "-p " : "",
 	   flags & DL_FL_FORCE ? "-f " : "",
-	   flags & DL_FL_UNLINK ? "-d " : "",
 	   flags & DL_FL_RETRY ? r : "",
 	   f);
   
-  FREE(&f);
-
-  return mutt_system(cmd);
+  return mutt_system (cmd);
 }
 
 #else 
@@ -252,7 +250,7 @@ int mx_lock_file (const char *path, int fd, int excl, int dot, int timeout)
   return 0;
 }
 
-int mx_unlock_file (const char *path, int fd, int dot)
+int mx_unlock_file (const char *path, int fd)
 {
 #ifdef USE_FCNTL
   struct flock unlockit = { F_UNLCK, 0, 0, 0 };
@@ -268,8 +266,7 @@ int mx_unlock_file (const char *path, int fd, int dot)
 #endif
 
 #ifdef USE_DOTLOCK
-  if (dot)
-    undotlock_file (path);
+  undotlock_file (path);
 #endif
   
   return 0;
@@ -292,32 +289,6 @@ FILE *mx_open_file_lock (const char *path, const char *mode)
   return (f);
 }
 
-void mx_unlink_empty (const char *path)
-{
-  int fd;
-#ifndef USE_DOTLOCK
-  char b;
-#endif
-
-  if ((fd = open (path, O_RDWR)) == -1)
-    return;
-  
-  if (mx_lock_file (path, fd, 1, 0, 1) == -1)
-  {
-    close (fd);
-    return;
-  }
-
-#ifdef USE_DOTLOCK
-  invoke_dotlock (path, DL_FL_UNLINK, 1);
-#else
-  if  (read (fd, &b, 1) != 1)
-    unlink (path);
-#endif
-
-  mx_unlock_file (path, fd, 0);
-}
-
 /* try to figure out what type of mailbox ``path'' is
  *
  * return values:
@@ -815,6 +786,10 @@ int mx_close_mailbox (CONTEXT *ctx)
       return (-1);
   }
 
+#ifdef USE_IMAP
+  /* IMAP doesn't support an OLD flag */
+  if (ctx->magic != M_IMAP)
+#endif
   if (option (OPTMARKOLD))
   {
     for (i = 0; i < ctx->msgcount; i++)
@@ -880,7 +855,7 @@ int mx_close_mailbox (CONTEXT *ctx)
   if (ctx->msgcount == ctx->deleted &&
       (ctx->magic == M_MMDF || ctx->magic == M_MBOX) &&
       !mutt_is_spool(ctx->path) && !option (OPTSAVEEMPTY))
-    mx_unlink_empty (ctx->path);
+    unlink (ctx->path);
 
   mx_fastclose_mailbox (ctx);
 
@@ -1040,8 +1015,70 @@ int mx_sync_mailbox (CONTEXT *ctx)
   return (rc);
 }
 
+int mh_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
+{
+  int hi = 0;
+  int fd, n;
+  char *cp;
+  char path[_POSIX_PATH_MAX];
+  DIR *dirp;
+  struct dirent *de;
 
-/* {maildir,mh}_open_new_message are in mh.c. */
+  do
+  {
+    if ((dirp = opendir (dest->path)) == NULL)
+    {
+      mutt_perror (dest->path);
+      return (-1);
+    }
+
+    /* figure out what the next message number is */
+    while ((de = readdir (dirp)) != NULL)
+    {
+      cp = de->d_name;
+      while (*cp)
+      {
+	if (!isdigit ((unsigned char) *cp))
+	  break;
+	cp++;
+      }
+      if (!*cp)
+      {
+	n = atoi (de->d_name);
+	if (n > hi)
+	  hi = n;
+      }
+    }
+    closedir (dirp);
+    hi++;
+    snprintf (path, sizeof (path), "%s/%d", dest->path, hi);
+    if ((fd = open (path, O_WRONLY | O_EXCL | O_CREAT, 0600)) == -1)
+    {
+      if (errno != EEXIST)
+      {
+	mutt_perror (path);
+	return (-1);
+      }
+    }
+  }
+  while (fd < 0);
+
+  if ((msg->fp = fdopen (fd, "w")) == NULL)
+    return (-1);
+
+  return 0;
+}
+
+int maildir_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
+{
+  char tmp[_POSIX_PATH_MAX];
+  char path[_POSIX_PATH_MAX];
+
+  maildir_create_filename (dest->path, hdr, path, tmp);
+  if ((msg->fp = safe_fopen (tmp, "w")) == NULL)
+    return (-1);
+  return 0;
+}
 
 int mbox_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
 {
@@ -1058,6 +1095,7 @@ int imap_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
   if ((msg->fp = safe_fopen (tmp, "w")) == NULL)
     return (-1);
   msg->path = safe_strdup(tmp);
+  msg->ctx = dest;
   return 0;
 }
 #endif
@@ -1175,31 +1213,30 @@ MESSAGE *mx_open_message (CONTEXT *ctx, int msgno)
 
     case M_MH:
     case M_MAILDIR:
-    {
-      HEADER *cur = ctx->hdrs[msgno];
-      char path[_POSIX_PATH_MAX];
-      
-      snprintf (path, sizeof (path), "%s/%s", ctx->path, cur->path);
-      if ((msg->fp = fopen (path, "r")) == NULL)
       {
-	mutt_perror (path);
-	dprint (1, (debugfile, "mx_open_message: fopen: %s: %s (errno %d).\n",
-		    path, strerror (errno), errno));
-	FREE (&msg);
+	HEADER *cur = ctx->hdrs[msgno];
+	char path[_POSIX_PATH_MAX];
+
+	snprintf (path, sizeof (path), "%s/%s", ctx->path, cur->path);
+	if ((msg->fp = fopen (path, "r")) == NULL)
+	{
+	  mutt_perror (path);
+	  dprint (1, (debugfile, "mx_open_message: fopen: %s: %s (errno %d).\n",
+		      path, strerror (errno), errno));
+	  FREE (&msg);
+	}
       }
-    }
-    break;
-    
+      break;
+
 #ifdef USE_IMAP
     case M_IMAP:
-    {
       if (imap_fetch_message (msg, ctx, msgno) != 0)
 	FREE (&msg);
       break;
-    }
 #endif /* USE_IMAP */
 
     default:
+
       dprint (1, (debugfile, "mx_open_message(): function not implemented for mailbox type %d.\n", ctx->magic));
       FREE (&msg);
       break;
@@ -1207,87 +1244,45 @@ MESSAGE *mx_open_message (CONTEXT *ctx, int msgno)
   return (msg);
 }
 
-/* commit a message to a folder */
-
-int mx_commit_message (MESSAGE *msg, CONTEXT *ctx)
-{
-  int r = 0;
-
-  if (!(msg->write && ctx->append))
-  {
-    dprint (1, (debugfile, "mx_commit_message(): msg->write = %d, ctx->append = %d\n",
-		msg->write, ctx->append));
-    return -1;
-  }
-
-  switch (msg->magic)
-  {
-    case M_MMDF:
-    {
-      fputs (MMDF_SEP, msg->fp);
-      break;
-    }
-    
-    case M_MBOX:
-    {
-      fputc ('\n', msg->fp);
-      break;
-    }
-
-    case M_KENDRA:
-    {
-      fputs (KENDRA_SEP, msg->fp);
-      break;
-    }
-
-#ifdef USE_IMAP
-    case M_IMAP:
-    {
-      if ((r = safe_fclose (&msg->fp)) == 0)
-	r = imap_append_message (ctx, msg);
-      break;
-    }
-#endif
-    
-    case M_MAILDIR:
-    {
-      if ((r = safe_fclose (&msg->fp)) == 0)
-	r = maildir_commit_message (ctx, msg, NULL);
-      break;
-    }
-    
-    case M_MH:
-    {
-      if ((r = safe_fclose (&msg->fp)) == 0)
-	r = mh_commit_message (ctx, msg, NULL);
-      break;
-    }
-  }
-  return r;
-}
-
 /* close a pointer to a message */
 int mx_close_message (MESSAGE **msg)
 {
   int r = 0;
 
-  if ((*msg)->magic == M_MH || (*msg)->magic == M_MAILDIR
-#ifdef USE_IMAP
-      || (*msg)->magic == M_IMAP
-#endif
-      )
+  if ((*msg)->write)
   {
-    r = safe_fclose (&(*msg)->fp);
-  }
-  else
-    (*msg)->fp = NULL;
+    /* add the message terminator */
+    switch ((*msg)->magic)
+    {
+      case M_MMDF:
+	fputs (MMDF_SEP, (*msg)->fp);
+	break;
 
-  if ((*msg)->path)
+      case M_MBOX:
+	fputc ('\n', (*msg)->fp);
+	break;
+    }
+  }
+
+  switch ((*msg)->magic) 
   {
-    dprint (1, (debugfile, "mx_close_message (): unlinking %s\n",
-		(*msg)->path));
-    unlink ((*msg)->path);
-    FREE (&(*msg)->path);
+    case M_MH:
+    case M_MAILDIR:
+      r = fclose ((*msg)->fp);
+      break;
+
+#ifdef USE_IMAP
+    case M_IMAP:
+      r = fclose ((*msg)->fp);
+      if ((*msg)->write && (*msg)->ctx->append)
+      {
+	r = imap_append_message ((*msg)->ctx, *msg);
+	unlink ((*msg)->path);
+      }
+#endif
+
+    default:
+      break;
   }
 
   FREE (msg);
