@@ -1,6 +1,5 @@
 /*
  * Copyright (C) 1996-1998 Michael R. Elkins <me@cs.hmc.edu>
- * Copyright (C) 1999 Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -44,15 +43,6 @@
 #include <string.h>
 #include <errno.h>
 
-static const char Mailbox_is_read_only[] = N_("Mailbox is read-only.");
-
-#define CHECK_READONLY if (Context->readonly) \
-{\
-    mutt_flushinp (); \
-    mutt_error _(Mailbox_is_read_only); \
-    break; \
-}
-
 static struct mapping_t AttachHelp[] = {
   { N_("Exit"),  OP_EXIT },
   { N_("Save"),  OP_SAVE },
@@ -134,8 +124,6 @@ ATTACHPTR **mutt_gen_attach_list (BODY *m,
       new->content = m;
       new->parent_type = parent_type;
       new->level = level;
-      /* called when creating new menu, so clear the tagged indicator */
-      m->tagged = 0;
 
       /* We don't support multipart messages in the compose menu yet */
       if (!compose && mutt_is_message_type(m->type, m->subtype))
@@ -151,7 +139,9 @@ ATTACHPTR **mutt_gen_attach_list (BODY *m,
   return (idx);
 }
 
-/* %D = deleted flag
+/* %c = character set: convert?
+ * %C = character set
+ * %D = deleted flag
  * %d = description
  * %e = MIME content-transfer-encoding
  * %f = filename
@@ -174,12 +164,38 @@ const char *mutt_attach_fmt (char *dest,
 {
   char fmt[16];
   char tmp[SHORT_STRING];
+  char charset[SHORT_STRING];
   ATTACHPTR *aptr = (ATTACHPTR *) data;
   int optional = (flags & M_FORMAT_OPTIONAL);
   size_t l;
   
   switch (op)
   {
+    case 'C':
+      if (!optional)
+      {
+	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
+	if (mutt_is_text_type (aptr->content->type, aptr->content->subtype) &&
+	    mutt_get_send_charset (charset, sizeof (charset), aptr->content, 0))
+	  snprintf (dest, destlen, fmt, charset);
+	else
+	  snprintf (dest, destlen, fmt, "");
+      }
+      else if (!mutt_is_text_type (aptr->content->type, aptr->content->subtype) ||
+	       !mutt_get_send_charset (charset, sizeof (charset), aptr->content, 0))
+        optional = 0;
+      break;
+    case 'c':
+      /* XXX */
+      if (!optional)
+      {
+	snprintf (fmt, sizeof (fmt), "%%%sc", prefix);
+	snprintf (dest, destlen, fmt, aptr->content->type != TYPETEXT ||
+		  aptr->content->noconv ? 'n' : 'c');
+      }
+      else if (aptr->content->type != TYPETEXT || aptr->content->noconv)
+        optional = 0;
+      break;
     case 'd':
       if(!optional)
       {
@@ -325,7 +341,7 @@ void attach_entry (char *b, size_t blen, MUTTMENU *menu, int num)
 
 int mutt_tag_attach (MUTTMENU *menu, int n)
 {
-  return (((ATTACHPTR **) menu->data)[n]->content->tagged = !((ATTACHPTR **) menu->data)[n]->content->tagged);
+  return ((((ATTACHPTR **) menu->data)[n]->content->tagged = !((ATTACHPTR **) menu->data)[n]->content->tagged) ? 1 : -1);
 }
 
 int mutt_is_message_type (int type, const char *subtype)
@@ -721,7 +737,7 @@ copy_tagged_attachments (FILE *fpout, FILE *fpin, const char *boundary, BODY *bd
   {
     if (bdy->tagged)
     {
-      fprintf (fpout, "\n--%s\n", boundary);
+      fprintf (fpout, "--%s\n", boundary);
       fseek (fpin, bdy->hdr_offset, 0);
       mutt_copy_bytes (fpin, fpout, bdy->length + bdy->offset - bdy->hdr_offset);
     }
@@ -766,7 +782,6 @@ create_tagged_message (const char *tempfile,
     mutt_copy_bytes (src->fp, msg->fp, body->length);
   }
 
-  mx_commit_message (msg, &tmpctx);
   mx_close_message (&msg);
   mx_close_message (&src);
   mx_close_mailbox (&tmpctx);
@@ -788,11 +803,7 @@ static void reply_attachment_list (int op, int tag, HEADER *hdr, BODY *body)
   if (!tag && body->hdr)
   {
     hn = body->hdr;
-    hn->msgno   = hdr->msgno; /* required for MH/maildir */
-    hn->replied = hdr->replied;
-    hn->read    = hdr->read;
-    hn->old	= hdr->old;
-    hn->changed = hdr->changed;
+    hn->msgno = hdr->msgno; /* required for MH/maildir */
     ctx = Context;
   }
   else
@@ -806,13 +817,8 @@ static void reply_attachment_list (int op, int tag, HEADER *hdr, BODY *body)
 
   ci_send_message (op, NULL, NULL, ctx, hn);
 
-  if (hn->replied)
-  {
-    if (Context != ctx)
-      mutt_set_flag (Context, hdr, M_REPLIED, 1);
-    else
-      _mutt_set_flag (Context, hdr, M_REPLIED, 1, 0);
-  }
+  if (hn->replied && !hdr->replied)
+    mutt_set_flag (Context, hdr, M_REPLIED, 1);
 
   if (ctx != Context)
   {
@@ -911,7 +917,7 @@ void mutt_view_attachments (HEADER *hdr)
 #ifdef _PGPPATH
   if((hdr->pgp & PGPENCRYPT) && !pgp_valid_passphrase())
   {
-    mx_close_message (&msg);
+    mx_close_message(&msg);
     return;
   }
   
@@ -990,7 +996,6 @@ void mutt_view_attachments (HEADER *hdr)
 	break;
 
       case OP_DELETE:
-	CHECK_READONLY;
 
 #ifdef _PGPPATH
         if (hdr->pgp)
@@ -1041,32 +1046,31 @@ void mutt_view_attachments (HEADER *hdr)
         break;
 
       case OP_UNDELETE:
-	CHECK_READONLY;
-	if (!menu->tagprefix)
-	{
-	  idx[menu->current]->content->deleted = 0;
-	  if (option (OPTRESOLVE) && menu->current < menu->max - 1)
-	  {
-	    menu->current++;
-	    menu->redraw = REDRAW_MOTION_RESYNCH;
-	  }
-	  else
-	    menu->redraw = REDRAW_CURRENT;
-	}
-	else
-	{
-	  int x;
+       if (!menu->tagprefix)
+       {
+	 idx[menu->current]->content->deleted = 0;
+	 if (option (OPTRESOLVE) && menu->current < menu->max - 1)
+	 {
+	   menu->current++;
+	   menu->redraw = REDRAW_MOTION_RESYNCH;
+	 }
+	 else
+	   menu->redraw = REDRAW_CURRENT;
+       }
+       else
+       {
+	 int x;
 
- 	  for (x = 0; x < menu->max; x++)
-	  {
-	    if (idx[x]->content->tagged)
-	    {
-	      idx[x]->content->deleted = 0;
-	      menu->redraw = REDRAW_INDEX;
-	    }
-	  }
-        }
-        break;
+	 for (x = 0; x < menu->max; x++)
+	 {
+	   if (idx[x]->content->tagged)
+	   {
+	     idx[x]->content->deleted = 0;
+	     menu->redraw = REDRAW_INDEX;
+	   }
+	 }
+       }
+       break;
 
       case OP_BOUNCE_MESSAGE:
         CHECK_ATTACH;
