@@ -38,6 +38,9 @@
 #include "pgp.h"
 #endif
 
+#ifdef MIXMASTER
+#include "remailer.h"
+#endif
 
 
 static void append_signature (FILE *f)
@@ -334,7 +337,7 @@ static int include_forward (CONTEXT *ctx, HEADER *cur, FILE *out)
   fputs (" -----\n\n", out);
   if (option (OPTFORWDECODE))
   {
-    cmflags |= M_CM_DECODE;
+    cmflags |= M_CM_DECODE | M_CM_CHARCONV;
     if (option (OPTFORWWEEDHEADER))
     {
       chflags |= CH_WEED;
@@ -352,7 +355,7 @@ static int include_forward (CONTEXT *ctx, HEADER *cur, FILE *out)
 static int include_reply (CONTEXT *ctx, HEADER *cur, FILE *out)
 {
   char buffer[STRING];
-  int flags = M_CM_PREFIX | M_CM_DECODE;
+  int flags = M_CM_PREFIX | M_CM_DECODE | M_CM_CHARCONV;
 
 
 
@@ -793,13 +796,19 @@ static int send_message (HEADER *msg)
   char tempfile[_POSIX_PATH_MAX];
   FILE *tempfp;
   int i;
-
+  
   /* Write out the message in MIME form. */
   mutt_mktemp (tempfile);
   if ((tempfp = safe_fopen (tempfile, "w")) == NULL)
     return (-1);
 
-  mutt_write_rfc822_header (tempfp, msg->env, msg->content, 0);
+#ifdef MIXMASTER
+  mutt_write_rfc822_header (tempfp, msg->env, msg->content, 0, msg->chain ? 1 : 0);
+#endif
+#ifndef MIXMASTER
+  mutt_write_rfc822_header (tempfp, msg->env, msg->content, 0, 0);
+#endif
+  
   fputc ('\n', tempfp); /* tie off the header. */
 
   if ((mutt_write_mime_body (msg->content, tempfp) == -1))
@@ -816,9 +825,14 @@ static int send_message (HEADER *msg)
     return (-1);
   }
 
+#ifdef MIXMASTER
+  if (msg->chain)
+    return mix_send_message (msg->chain, tempfile);
+#endif
+
   i = mutt_invoke_sendmail (msg->env->to, msg->env->cc, msg->env->bcc,
 		       tempfile, (msg->content->encoding == ENC8BIT));
-  return (i ? -1 : 0);
+  return (i);
 }
 
 /* rfc2047 encode the content-descriptions */
@@ -1189,11 +1203,13 @@ main_loop:
 #ifdef _PGPPATH
   if (msg->pgp)
   {
+    if (pgp_get_keys (msg, &pgpkeylist) == -1)
+      goto main_loop;
+
     /* save the decrypted attachments */
     save_content = msg->content;
 
-    if ((pgp_get_keys (msg, &pgpkeylist) == -1) ||
-	(pgp_protect (msg, pgpkeylist) == -1))
+    if (pgp_protect (msg, pgpkeylist) == -1)
     {
       if (msg->content->parts)
       {
@@ -1311,7 +1327,7 @@ full_fcc:
   }
 #endif /* _PGPPATH */
 
-  if (send_message (msg) == -1)
+  if ((i = send_message (msg)) == -1)
   {
     if (!(flags & SENDBATCH))
     {
@@ -1324,9 +1340,8 @@ full_fcc:
       goto cleanup;
     }
   }
-
-  if (!option (OPTNOCURSES) && ! (flags & SENDMAILX))
-    mutt_message _("Mail sent.");
+  else if (!option (OPTNOCURSES) && ! (flags & SENDMAILX))
+    mutt_message (i == 0 ? _("Mail sent.") : _("Sending in background."));
 
   if (flags & SENDREPLY)
   {
